local TITLE = 'Worldgen'

local SHADER_SRC_PATH = 'shader.txt'
local shader

local MOVE_SPEED = 400
local ZOOM_SPEED = 5
local x
local y
local zoom

local INSTRUCTIONS
  = 'Move with arrow keys.  W/= to zoom in.  S/- to zoom out.'
local INSTRUCTIONS_X = 10
local INSTRUCTIONS_Y = 10
local TEXT_SHADOW = 3

local function file_to_string(file_path)
    local file = love.filesystem.newFile(file_path)
    file:open('r')

    local result = file:read()
    file:close()

    return result
end

function love.load()
    shader = love.graphics.newPixelEffect(file_to_string(
        SHADER_SRC_PATH
    ))

    x = 0
    y = 0
    zoom = 1
    --shader:send('seed', os.time())

    shader:send('width', love.graphics.getWidth())
    shader:send('height', love.graphics.getHeight())
    x = love.graphics.getWidth() / 2
    y = love.graphics.getHeight() / 2
end

function love.keypressed(key)
    if key == 'escape' then
        love.event.push('quit')
    -- elseif key == 'space' then
    --     shader:send('seed', os.time())
    end
end

function love.update(dt)
    if love.keyboard.isDown('up') then
        y = y + ((MOVE_SPEED / zoom) * dt)
    elseif love.keyboard.isDown('down') then
        y = y - ((MOVE_SPEED / zoom) * dt)
    end

    if love.keyboard.isDown('left') then
        x = x - ((MOVE_SPEED / zoom) * dt)
    elseif love.keyboard.isDown('right') then
        x = x + ((MOVE_SPEED / zoom) * dt)
    end

    if love.keyboard.isDown('=') or love.keyboard.isDown('w') then
        zoom = zoom + (ZOOM_SPEED * dt)
    elseif love.keyboard.isDown('-') or love.keyboard.isDown('s') then
        zoom = math.max(1, zoom - (ZOOM_SPEED * dt))
    end

    shader:send('x', x)
    shader:send('y', y)
    shader:send('zoom', zoom)

    love.graphics.setCaption(TITLE .. ' - FPS: '
      .. love.timer.getFPS())
end

function love.draw()
    love.graphics.setPixelEffect(shader)
    love.graphics.rectangle(
        'fill',
        0, 0,
        love.graphics.getWidth(), love.graphics.getHeight()
    )

    love.graphics.setPixelEffect()

    love.graphics.setColor(0, 0, 128)
    for i = 1, TEXT_SHADOW do
        love.graphics.print(
            INSTRUCTIONS, INSTRUCTIONS_X + (i / 2),
            INSTRUCTIONS_Y + (i / 2)
        )
    end
    love.graphics.setColor(255, 255, 255)
    love.graphics.print(
        INSTRUCTIONS, INSTRUCTIONS_X, INSTRUCTIONS_Y
    )
end